<?php

use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;
use Slim\Factory\AppFactory;

use DI\Container;
use Slim\Views\Twig;
use Slim\Views\TwigExtension;
use Slim\Views\TwigMiddleware;

require __DIR__ . '/../vendor/autoload.php';
// Create Container
$container = new Container();
AppFactory::setContainer($container);

/**
 * Instantiate App
 *
 * In order for the factory to work you need to ensure you have installed
 * a supported PSR-7 implementation of your choice e.g.: Slim PSR-7 and a supported
 * ServerRequest creator (included with Slim PSR-7)
 */
$app = AppFactory::create();

// Add Routing Middleware
$app->addRoutingMiddleware();

/**
 * Add Error Handling Middleware
 *
 * @param bool $displayErrorDetails -> Should be set to false in production
 * @param bool $logErrors -> Parameter is passed to the default ErrorHandler
 * @param bool $logErrorDetails -> Display error details in error log
 * which can be replaced by a callable of your choice.
 
 * Note: This middleware should be added last. It will not handle any exceptions/errors
 * for middleware added after it.
 */
$errorMiddleware = $app->addErrorMiddleware(true, true, true);


// Add Twig-View Middleware
$container->set('view', function () {
    // Of course put correct path to your views here
    return Twig::create('../templates', ['cache' => false]);
});

$app->add(TwigMiddleware::createFromContainer($app));
class TimeSheetDB extends SQLite3
{

    function __construct()
    {
        $this->open('../../db/timesheet.db');
    }

    function newTask($name, $description, $user, $status, $eta, $priority, $deadline, $parent)
    {
        $query = $this->prepare('insert into tasks (name, description, user_ref, status, eta, priority, deadline, parent_ref) 
        values (:name, :description, :user_ref, :status, :eta, :priority, :deadline, :parent_ref)');

        $query->bindValue(':name', $name);
        $query->bindValue(':description', $description);
        $query->bindValue(':user_ref', $user);
        $query->bindValue(':status', $status);
        $query->bindValue(':eta', $eta);
        $query->bindValue(':priority', $priority);
        $query->bindValue(':deadline', $deadline);
        $query->bindValue(':parent_ref', $parent);

        $query->execute();
    }

    function newUser($userName, $password, $name, $email, $manager)
    {
        $query = $this->prepare('insert into users (username, password, name, email, manager_ref)
        values (username, password, name, email, manager_ref)');

        $query->bindValue(':username', $userName);
        $query->bindValue(':password', $password);
        $query->bindValue(':name', $name);
        $query->bindValue(':email', $email);
        $query->bindValue(':manager', $manager);

        $query->execute();
    }
    function getTasks()
    {
        $query = $this->prepare('SELECT * from tasks');
        $dbResults = $query->execute();
        $resultArray = array();

        while ($row = $dbResults->fetchArray()) {
            array_push($resultArray, $row);
        }
        return $resultArray;
    }
    function newTimeline($task_ref, $user_ref, $start_dt, $end_dt, $duration,$note)
    {
        $query = $this->prepare ('INSERT INTO timelines  (task_ref,user_ref,start_dt,end_dt,duration,note) 
        VALUES (:task_ref,:user_ref,:start_dt,:end_dt,:duration,note)');

        $query->bindValue(':task_ref', $task_ref);
        $query->bindValue(':user_ref', $user_ref);
        $query->bindValue(':start_dt', $start_dt);
        $query->bindValue(':end_dt', $end_dt);
        $query->bindValue(':duration', $duration);
        $query->bindValue(':note', $note);



        $query->execute();
    }
    
}

$container->set('db', function () {
    return new TimeSheetDB();
});

// Define app routes
$app->get('/hello/{name}', function (Request $request, Response $response, $args) {
    $name = $args['name'];
    $response->getBody()->write("Hello, $name");
    return $response;
});

$app->get('/task/new', function (Request $request, Response $response, $args) {
    return $this->get('view')->render($response, 'newTask.twig', []);
});

$app->post('/task/save', function (Request $request, Response $response, $args) {
    $name = $_POST['name'];
    $description = $_POST['description'];
    $userRef = $_POST['user_ref'];
    $status = $_POST['status'];
    $eta = $_POST['eta'];
    $priority = $_POST['priority'];
    $deadline = $_POST['deadline'];
    $parentRef = $_POST['parent_ref'];

    $db = $this->get('db');
    $db->newTask($name, $description, $userRef, $status, $eta, $priority, $deadline, $parentRef);

    return $response->withStatus(302)->withHeader('Location', $_SERVER['HTTP_REFERER']);
});

$app->get('/task/overview[/{user}]', function (Request $request, Response $response, $args) {
    if ($_GET) {
    }

    if ($args['user']) {
    }

    $db = $this->get('db');
    $data = $db->getTasks();


    return $this->get('view')->render($response, 'overview.twig', [
        'data' => $data
    ]);
});

// Run app
$app->run();
